#ifndef __XMLTLK_UTIL_H
#define __XMLTLK_UTIL_H

#define ONEK 1024
#define ATOMICDELETE(ptr) { if ((ptr) != NULL) { delete (ptr); (ptr) = NULL; } }
#define ATOMICFREE(ptr) { if ((ptr) != NULL) { free(ptr); (ptr) = NULL; } }
#define ATOMICRELEASE(punk) { if ((punk) != NULL) { (punk)->Release(); (punk) = NULL; } }
#define ASSIGN(pDest, pSource) { (pDest) = (pSource); (pSource)->AddRef(); }
#define ARRAYSIZE(A)    (sizeof(A)/sizeof((A)[0]))
#define min(a, b)  (((a) < (b)) ? (a) : (b))
#define max(a, b)  (((a) > (b)) ? (a) : (b))

#define CHECK_ALLOC(ptr) { if ((ptr) == NULL) { throw CXmlToolkitException(ERROR_OUT_OF_MEMORY); } }

class XString
{
public:
    char * pszString;
    size_t cchString;

    XString(): pszString(NULL), cchString(0) {}
    XString(char * psz, size_t cch): pszString(psz), cchString(cch) {}
    
    XString * Copy();
    static int Compare(XString * pxs1, XString * pxs2);
};

class CStringBuffer
{
public:
    // CStringBuffer constructor and destructor.
    CStringBuffer();
    ~CStringBuffer();

    // Write text to the string buffer.
    void Write(char * pszText, int cchText);

    // Return the accumulated string from the string buffer.
    // WARNING: CStringBuffer has to be in scope while manipulating this string!!
    const char * SzString();

    // Clears out the string buffer, so that it can be reused.
    void Clear();

    // Total size of the allocated buffer.
    size_t GetSize() { return m_cchBuffer; }
    
private:
    // This is the initial size of the stack-based buffer.
    static const int INIT_BUFFER_SIZE = 100;
    
    bool   m_fStack;                          // Is the string buffer in static or dynamic mode?
    char * m_pchCurrent;                      // Position where the next character can be written.
    char   m_szBuffer[INIT_BUFFER_SIZE + 1];  // Static stack-based buffer. Always leave room for the null terminator.
    char * m_pszBuffer;                       // Dynamic buffer.
    int    m_cchBuffer;                       // Number of bytes allocated for the dynamic buffer.
};

#endif // __XMLTLK_UTIL_H