#include "xmltk.h"

typedef struct
{
    char * pszRoot;
    int iIndentSize;
    int iFileArg;
} XCatArgs;

bool FProcessInput(int argc, char * argv[], XCatArgs * pArgs)
{
    if (argc < 2)
        return false;

    for (int iArg = 1 ; iArg < argc ; iArg++)
    {
        char * pszArg = argv[iArg];
        if (*pszArg++ != '-')
        {
            pArgs->iFileArg = iArg;
            return true;
        }
        
        switch (*pszArg)
        {
        case 'r':
            pArgs->pszRoot = argv[++iArg];
            break;
        case 'i':
            pArgs->iIndentSize = atol(argv[++iArg]);
            break;
        default:
            return false;
        }
    }

    return false;
}

int main(int argc, char *argv[])
{
    XCatArgs args;
    memset(&args, 0, sizeof(XCatArgs));
    
    if (!FProcessInput(argc, argv, &args))
    {
        cout << "Usage: xcat [-r <root>] [-i <indentation>] (<xml file>)+\n";
        cout << "  [-r <root>]          Optional root element to wrap the result with.\n";
        cout << "  [-i <indentation>]   Optional indentation size, 0 if not specified.\n";
        cout << "  (<xml file>)+        XML files to concatenate.\n";
                
        return 0;
    }

    InitGlobalTokenTable();
    try
    {
        IXmlWriter * pchOut = CreateXmlWriter(args.iIndentSize);
        pchOut->SetOutputFile(stdout);

        if (args.pszRoot != NULL)
        {
            pchOut->startElement(g_ptt->XTOKENFromStr(args.pszRoot, XST_ELEMENT));
        }

        for (int iFileArg = args.iFileArg ; iFileArg < argc; iFileArg++)
        {
            CXmlToolkit xmltk(argv[iFileArg], pchOut);
            xmltk.Parse();
        }

        if (args.pszRoot != NULL)
        {
            pchOut->endElement(g_ptt->XTOKENFromStr(args.pszRoot, XST_ELEMENT));
        }

        ATOMICRELEASE(pchOut);
    }
    catch (CXmlToolkitException e)
    {
        printf("Error: %s", e.PszMessage());
    }
    CleanupGlobalTokenTable();

    return 0;
}
