// -*- mode: c++ -*-
//  copyright (C) 2001 Makoto Onizuka, University of Washington
//  $Id: Query.cxx,v 1.3 2002/08/13 12:59:46 monizuka Exp $

#define QUERY_EMBODY
#ifndef WIN32
#include <iostream.h>
#else
#include <iostream>
#endif

#include "Query.h"
#include "Root.h"

Query::Query(void) : Base(),state(S_CONSTRUCT),type(T_STRING),query(0),top(0)
{
    parseTree = new Root();
	combinedAutomaton = 0;
	automata = 0;
}

Variable *
Query::parse(void)
{
    if (state == S_START_DOC || state == S_END_DOC)
    {
        throw CXmlToolkitException(errmes[E_STATE]);
    }
  	
    __gQuery = this;
	state = S_PARSED;
	return parseTree->parse();
}

void 
Query::xpath2NFA(void){
  	VarPtrList * vs = parseTree->getVarPtrMap();
	automata  = new AutomataPtrList();
	for (VarPtrList::iterator itr = vs->begin(); itr != vs->end(); ++itr){
	  Automata * a = new Automata();
	  a->create((*itr)->getXPath(), automata);
	  automata->push_back(a);
	}
}

void 
Query::NFAs2DFAs(void){
	for (AutomataPtrList::iterator itr = automata->begin(); itr != automata->end(); ++itr){
	  (*itr)->NFA2DFA();
	}
}

//
// for SAX interfaces
//
Query::FAStates
Query::startDocument(void)
{
    if (state != S_PARSED)
    {
        throw CXmlToolkitException(errmes[E_STATE]);
    }
    
    state = S_START_DOC;
	switch (combinedAutomaton->startDocument())
    {
	case Automata::S_INACTIVE:
        return S_INACTIVE;
	case Automata::S_COMPLETE:
        return S_COMPLETE;
	default:
        return S_SKIP;
	}
}

Query::FAStates
Query::endDocument(void)
{
    if (state != S_START_DOC)
        throw CXmlToolkitException(errmes[E_STATE]);

    state = S_END_DOC;
  	switch (combinedAutomaton->endDocument())
    {
	case Automata::S_INACTIVE:
        return S_INACTIVE;
	case Automata::S_COMPLETE:
        return S_COMPLETE;
	default:
        return S_SKIP;
	}
}

Query::FAStates
Query::startElement(PString * ps){
#if defined(DEBUG)
  char * debug = ps->getString(ps);
#endif
	switch (combinedAutomaton->startElement(ps)){
	case Automata::S_INACTIVE:
	  return S_INACTIVE;
	case Automata::S_COMPLETE:
	  return S_COMPLETE;
	default:					// case Automata::S_SKIP:
	  return S_SKIP;
	}
}

Query::FAStates
Query::endElement(PString * ps){
#if defined(DEBUG)
  char * debug = ps->getString(ps);
#endif
	switch (combinedAutomaton->endElement()){
	case Automata::S_INACTIVE:
	  return S_INACTIVE;
	case Automata::S_COMPLETE:
	  return S_COMPLETE;
	default:		// case Automata::S_SKIP;
	  return S_SKIP;
	}
}

Query::FAStates Query::endElement(void)
{
	switch (combinedAutomaton->endElement())
    {
    case Automata::S_INACTIVE:
        return S_INACTIVE;
    case Automata::S_COMPLETE:
        return S_COMPLETE;
    default:
        return S_SKIP;
	}
}