// -*- mode: c++ -*-
//  copyright (C) 2001 Makoto Onizuka, University of Washington
//  $Id: Root.cxx,v 1.3 2002/09/09 06:45:56 monizuka Exp $

#include "Error.h"
#include "Root.h"
#include "List.cxx"
#include "Query.h"

extern int yyparse(void);
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;

template class List<Variable>;

// Error messages for the Root class.
char * Root::errmes[] = {
  "",
  "Yacc parse error",
  "There are more than two root node", 
  "State error",
  "Other class error",
  "System internal error"
};

Root::Root(const char * const fname) : Node(), filename(fname), state(S_CONSTRACT) 
{
    variables   = new VarPtrList();
    elements    = new List<Node>();
}

Root::Root(void) : Node(),filename(0),state(S_CONSTRACT)
{
    variables   = new VarPtrList();
    elements    = new List<Node>();
}

Root::~Root(){
    if (state != S_CONSTRACT)
        throw CXmlToolkitException(ERROR_UNEXPECTED);

    for (VarPtrList::iterator itr = variables->begin(); itr != variables->end(); ++itr){
        delete *itr;
	}
    delete variables;
	for (unsigned int i = 0; i <elements->getCount();i++){
	  delete elements->getItem(i);
	}
    delete elements;
}

// register Element
// If there is a registerd Element with the same,
Node *			// this function returns the element.
Root::registerElement(char * label, nodeType t){
    Node * n;
    unsigned long count = elements->getCount();
    for (unsigned long i = 0; i < count; i++){
      n = elements->getItem(i);
      if (strcmp(label, n->getNodeName()) == 0){
	if (n->getNodeType() == Node::TextNode)
	  n->setNodeType(t);
	return n;
      }
    }
    n = new Node(label, t);
    elements->insertItem(n);
    return n;
}

Variable *						// return last Variable
Root::parse(void)
{
    if (yyparse())
    {
        throw CXmlToolkitException("Syntax error.");
	}
	
    VarPtrList::reverse_iterator itr = variables->rbegin();
	return *itr;
}

void
Root::open(void)
{
    if (state != S_CONSTRACT)
        throw CXmlToolkitException(ERROR_UNEXPECTED);
	
    yyin = fopen(filename , "r");
    if (yyin == NULL)
        throw CXmlToolkitException(ERROR_INPUT_FILE);
	
    state = S_OPEN;
}

void
Root::close(void)
{
    if (state != S_OPEN)
        throw CXmlToolkitException(ERROR_UNEXPECTED);
    if (fclose(yyin) != 0)
        throw CXmlToolkitException(ERROR_UNEXPECTED);
    state = S_CONSTRACT;
}

void				// print for debugging
Root::printQuery(ofstream * ofs, char * fname){
    *ofs << "<?xml version=\"1.0\" encoding=\"euc-jp\"?>" << endl << endl;
    *ofs << "<query fileName=\"" << fname << "\">" << endl;
    *ofs << "  <variables count=\"" << variables->size() << "\">" << endl;
    *ofs << "  </variables>" << endl;
    *ofs << "  <nodes count=\"" << elements->getCount() << "\">" << endl;
    for (unsigned long i = 0; i < elements->getCount(); i++){
      Node * n = elements->getItem(i);
      *ofs << "    <node id=\"" << i << "\" name=\"" << n->getNodeName() << "\"/>" << endl;
    }
    *ofs << "  </nodes>" << endl;
    *ofs << "  <structure>" << endl;
    Node * n = elements->top();
    if (n != 0) n->print(ofs);
    *ofs << "  </structure>" << endl;
    *ofs << "</query>" << endl;
}

				// 1. Attribute uniqueness is checked by 
				//    Node::insert(CNodeDef *).
void				// 2. permit (A AS {a0}, A AS {a1}, B).
Root::checkTree(void){
    for (unsigned long i = 0; i < elements->getCount(); i++){
      Node * n = elements->getItem(i);
      n->check();
    }
}
